Module:    wt-example2
Synopsis:  An example of using the WildTangent WebDriver from Dylan
Author:    Chris Double
Copyright: (C) 2000, Chris Double.  All rights reserved.

// NOTE: Running this application within the Functional Developer IDE
// causes a 'floating point underflow' exception to occur. 
//
// To prevent this, set up the Functional Developer debugger to ignore
// floating point errors, rather than debug them. This can be done by
// choosing Application/Debug from the project menu. In the debugger,
// choose View/Debugger Options. In the exceptions tab of the resulting
// dialog, change 'Float overflow' and 'Float underflow' to be 'Report'
// or 'Ignore'. The program can then be run from the IDE as normal.

// I should really put this in a com utilities library.
define macro let-com-interface
  { let-com-interface ?:name :: ?type:expression = ?value:expression;
    ?:body }
  => { let ?name :: ?type = ?value;
       block()         
         ?body
       cleanup
         release(?name)
       end }
end macro let-com-interface;

define function as-com-type(type, interface-ptr)
  let (result, com-interface) = QueryInterface(interface-ptr, 
    dispatch-client-uuid(type));  
  if(SUCCEEDED?(result))
    pointer-cast(type, com-interface);
  else
    #f
  end if;
end function as-com-type;  

define frame <wt-example2-frame> (<simple-frame>)
  pane wild-tangent-pane (frame)
    make(<wild-tangent-gadget>, min-width: 640, min-height: 480);

  pane go-button (frame)
    make(<push-button>, label: "Go", activate-callback: on-go-button);

  pane stop-button (frame)
    make(<push-button>, label: "Stop", activate-callback: on-stop-button);

  layout (frame)
    vertically()
      horizontally()
        frame.go-button;
        frame.stop-button;
      end;
      frame.wild-tangent-pane;
    end;

  keyword title: = "WildTangent Example 2";
end frame <wt-example2-frame>;

define method on-go-button(g)
  let frame = g.sheet-frame;
  let wt-gadget = frame.wild-tangent-pane;
  let-com-interface scene :: <IWT> = wt-gadget.wild-tangent-interface;
  let-com-interface stage :: <IWTStage> = IWT/createStage(scene);
  let-com-interface camera :: <IWTCamera> = IWTStage/createCamera(stage);
//  IWTCamera/setViewRect(camera, 0, 0, 200, 200);
  IWTCamera/setPosition(camera, 0.0, 0.0, -2.0);

  let-com-interface ambient-light :: <IWTLight> = IWT/createLight(scene, $tWTLightAmbient);
  let-com-interface point-light :: <IWTLight> = IWT/createLight(scene, $tWTLightPoint);

  // This explicit query-interfacing here is required as the 
  // WildTangent libraries don't have the IWTLight interface derived from 
  // the IWTContainer interface. QueryInterface is required to get the
  // interfaces.
  let-com-interface al-as-container :: <IWTContainer> = as-com-type(<IWTContainer>, ambient-light);
  let-com-interface pl-as-container :: <IWTContainer> = as-com-type(<IWTContainer>, point-light);

  IWTStage/addObject(stage, al-as-container);
  IWTStage/addObject(stage, pl-as-container);

  IWTLight/setColor(ambient-light, 100, 100, 100);
  IWTLight/setColor(point-light, 150, 150, 150);
  IWTLight/setPosition(point-light, 100.0, 100.0, -100.0);


  let-com-interface model :: <IWTModel> = IWT/createBox(scene, 0.5, 0.5, 0.5, 5);
  let model-as-object :: <IWTObject> = as-com-type(<IWTObject>, model);

  let-com-interface container :: <IWTContainer> = IWT/createContainer(scene);
  IWTContainer/attach(container, model-as-object);

  IWTStage/addObject(stage, container);


  IWTModel/setColor(model, 255, 0, 0, "top");
  IWTModel/setColor(model, 0, 255, 0, "bottom");
  IWTModel/setColor(model, 0, 0, 255, "left");
  IWTModel/setColor(model, 255, 0, 255, "right");
  IWTModel/setColor(model, 255, 255, 0, "front");
  IWTModel/setColor(model, 0, 255, 255, "back");
  IWTContainer/setConstantRotation(container, 1.0, 1.0, 1.0, 120.0);

  IWT/start(scene);

end method on-go-button;

define method on-stop-button(g)
  let frame = g.sheet-frame;
  let wt-gadget = frame.wild-tangent-pane;
  let-com-interface scene :: <IWT> = wt-gadget.wild-tangent-interface;
  IWT/stop(scene);
end method on-stop-button;

define method main () => ()
  unless(wild-tangent-initialize())
    notify-user("Failed to load Wild Tangent Gadget libary. Check ATL.DLL version.");
  end;
  start-frame(make(<wt-example2-frame>));
  wild-tangent-uninitialize();
end method main;

begin
  main();
end;

